@@ -168,6 +168,7 @@ struct _GsfXMLInDoc {
 	GsfXMLInNode	*root;
 	GsfXMLInNS	*ns;
 	GPtrArray	*ns_by_id;
+	GsfXMLInUnknownFunc	unknown_handler;
 };
 
 typedef struct {
@@ -198,6 +199,7 @@ gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **
 	xmlChar const **ns_ptr;
 	char const *tmp;
 	int i;
+	gboolean check_unknown_handler = TRUE;
 
 	/* Scan for namespace declarations.  Yes it is ugly to have the api
 	 * flag that its children can declare namespaces. However, given that a
@@ -238,6 +240,7 @@ gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **
 		}
 	}
 
+lookup_child :
 	for (ptr = state->node->groups ; ptr != NULL ; ptr = ptr->next) {
 		group = ptr->data;
 		/* does the namespace match */
@@ -275,6 +278,12 @@ gsf_xml_in_start_element (GsfXMLIn *state, xmlChar const *name, xmlChar const **
 		}
 	}
 
+	if (check_unknown_handler) {
+		check_unknown_handler = FALSE; /* only loop once */
+		if (state->doc->unknown_handler != NULL &&
+		    (state->doc->unknown_handler) (state, name, attrs))
+			goto lookup_child;
+	}
 	if (state->unknown_depth++)
 		return;
 	g_warning ("Unexpected element '%s' in state %s.", name, node_name (state->node));
@@ -448,8 +457,6 @@ GsfXMLInDoc *
 gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
 {
 	GsfXMLInDoc  *doc;
-	GHashTable   *symbols;
-	GsfXMLInNode *tmp, *real_node, *node;
 	unsigned i;
 
 	if (root->parent_initialized)
@@ -468,10 +475,24 @@ gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
 			g_ptr_array_index (doc->ns_by_id, ns[i].ns_id) = ns+i;
 		}
 	}
+	gsf_xml_in_doc_extend (doc, root);
+	return doc;
+}
 
-	symbols = g_hash_table_new (g_str_hash, g_str_equal);
-	for (node = root; node->id != NULL ; node++ ) {
-		g_return_val_if_fail (!node->parent_initialized, NULL);
+void
+gsf_xml_in_doc_extend (GsfXMLInDoc  *doc,
+		       GsfXMLInNode *extension_nodes)
+{
+	GsfXMLInNode *tmp, *real_node, *node;
+	GHashTable   *symbols = g_hash_table_new (g_str_hash, g_str_equal);
+
+	g_return_if_fail (doc != NULL);
+	g_return_if_fail (extension_nodes != NULL);
+
+	if (extension_nodes->parent_initialized)
+		return;
+	for (node = extension_nodes; node->id != NULL ; node++ ) {
+		g_return_if_fail (!node->parent_initialized);
 
 		tmp = g_hash_table_lookup (symbols, node->id);
 		if (tmp != NULL) {
@@ -480,7 +501,7 @@ gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
 			    node->has_content != GSF_XML_NO_CONTENT || node->user_data.v_int != 0) {
 				g_warning ("ID '%s' has already been registered.\n"
 					   "The additional decls should not specify start,end,content,data", node->id);
-				return NULL;
+				return;
 			}
 			real_node = tmp;
 		} else {
@@ -512,7 +533,7 @@ gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
 			group->elem = g_slist_prepend (group->elem, real_node);
 		} else if (strcmp (node->id, node->parent_id)) {
 			g_warning ("Parent ID '%s' unknown", node->parent_id);
-			return NULL;
+			return;
 		}
 
 		/* WARNING VILE HACK :
@@ -528,8 +549,21 @@ gsf_xml_in_doc_new (GsfXMLInNode *root, GsfXMLInNS *ns)
 	}
 
 	g_hash_table_destroy (symbols);
+}
 
-	return doc;
+/**
+ * gsf_xml_in_doc_set_unknown_handler:
+ * @doc : #GsfXMLInDoc
+ * @handler : The function to call
+ *
+ * Call the function @handler when an unexpected child node is found
+ **/
+void
+gsf_xml_in_doc_set_unknown_handler (GsfXMLInDoc *doc,
+				    GsfXMLInUnknownFunc handler)
+{
+	g_return_if_fail (doc != NULL);
+	doc->unknown_handler = handler;
 }
 
 /**
@@ -779,15 +813,15 @@ gsf_xml_out_end_element (GsfXMLOut *xml)
 	xml->indent--;
 	switch (xml->state) {
 	case GSF_XML_OUT_NOCONTENT :
-	gsf_output_write (xml->output, 3, "/>\n");
-	break;
+		gsf_output_write (xml->output, 3, "/>\n");
+		break;
 
 	case GSF_XML_OUT_CHILD :
-	gsf_xml_out_indent (xml);
+		gsf_xml_out_indent (xml);
 	/* fall through */
 	case GSF_XML_OUT_CONTENT :
-	gsf_output_printf (xml->output, "</%s>\n", id);
-	};
+		gsf_output_printf (xml->output, "</%s>\n", id);
+	}
 	xml->state = GSF_XML_OUT_CHILD;
 	return id;
 }
@@ -852,14 +886,17 @@ gsf_xml_out_simple_float_element (GsfXMLOut *xml, char const *id,
  *
  * dump @val_utf8 to an attribute named @id without checking to see if the
  * content needs escaping.  A useful performance enhancement when the
- * application knows that structure of the content well.
+ * application knows that structure of the content well.  If @val_utf8 is NULL
+ * do nothing (no warning, no output)
  **/
 void
 gsf_xml_out_add_cstr_unchecked (GsfXMLOut *xml, char const *id,
 				char const *val_utf8)
 {
 	g_return_if_fail (xml != NULL);
-	g_return_if_fail (xml->state == GSF_XML_OUT_NOCONTENT);
+
+	if (val_utf8 == NULL)
+		return;
 
 	if (id == NULL) {
 		xml->state = GSF_XML_OUT_CONTENT;
@@ -876,7 +913,8 @@ gsf_xml_out_add_cstr_unchecked (GsfXMLOut *xml, char const *id,
  * @val_utf8 : a utf8 encoded string
  *
  * dump @val_utf8 to an attribute named @id or as the nodes content escaping
- * characters as necessary.
+ * characters as necessary.  If @val_utf8 is NULL do nothing (no warning, no
+ * output)
  **/
 void
 gsf_xml_out_add_cstr (GsfXMLOut *xml, char const *id,
@@ -885,6 +923,11 @@ gsf_xml_out_add_cstr (GsfXMLOut *xml, char const *id,
 	guint8 const *cur   = val_utf8;
 	guint8 const *start = val_utf8;
 
+	g_return_if_fail (xml != NULL);
+
+	if (val_utf8 == NULL)
+		return;
+
 	if (id == NULL) {
 		xml->state = GSF_XML_OUT_CONTENT;
 		gsf_output_write (xml->output, 1, ">");
@@ -911,6 +954,15 @@ gsf_xml_out_add_cstr (GsfXMLOut *xml, char const *id,
 				gsf_output_write (xml->output, cur-start, start);
 			start = ++cur;
 			gsf_output_write (xml->output, 6, "&quot;");
+		} else if (*cur < 0x20 && id != NULL) {
+			guint8 buf[8];
+			sprintf (buf, "&#%d;", *cur);
+
+			if (cur != start)
+				gsf_output_write (xml->output, cur-start, start);
+			start = ++cur;
+
+			gsf_output_write (xml->output, strlen (buf), buf);
 		} else if (((*cur >= 0x20) && (*cur != 0x7f)) ||
 			   (*cur == '\n') || (*cur == '\r') || (*cur == '\t')) {
 			cur++;
