@@ -27,8 +27,25 @@
 #include <stdio.h>
 #include <string.h>
 
+/*
+ * Glib gets this wrong, really.  ARM's floating point format is a weird
+ * mixture.
+ */
+#define G_ARMFLOAT_ENDIAN 56781234
+#if defined(__arm__) && !defined(__vfp__) && (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+#define G_FLOAT_BYTE_ORDER G_ARMFLOAT_ENDIAN
+#else
+#define G_FLOAT_BYTE_ORDER G_BYTE_ORDER
+#endif
+
+
 static void base64_init (void);
 
+/**
+ * gsf_init :
+ *
+ * Initializes the GSF library
+ */
 void
 gsf_init (void)
 {
@@ -36,6 +53,11 @@ gsf_init (void)
 	base64_init ();
 }
 
+/**
+ * gsf_shutdown:
+ * 
+ * De-intializes the GSF library
+ */
 void
 gsf_shutdown (void)
 {
@@ -74,6 +96,13 @@ gsf_mem_dump (guint8 const *ptr, size_t len)
 	gsf_mem_dump_full (ptr, len, 0);
 }
 
+/**
+ * gsf_input_dump :
+ * @input: a #GsfInput
+ * @dump_as_hex: If TRUE, dump in hexidecmal format
+ *
+ * Dumps @input's contents to STDOUT, optionally in hex format.
+ */
 void
 gsf_input_dump (GsfInput *input, gboolean dump_as_hex)
 {
@@ -95,6 +124,8 @@ gsf_input_dump (GsfInput *input, gboolean dump_as_hex)
 		size -= count;
 		offset += count;
 	}
+	if (!dump_as_hex)
+		fflush (stdout);
 }
 
 guint64
@@ -113,7 +144,7 @@ gsf_le_get_guint64 (void const *p)
 
 		return li;
 	} else {
-		g_error ("Big endian machine, but weird size of float");
+		g_error ("Big endian machine, but weird size of guint64");
 	}
 #elif G_BYTE_ORDER == G_LITTLE_ENDIAN
 	if (sizeof (guint64) == 8) {
@@ -125,7 +156,7 @@ gsf_le_get_guint64 (void const *p)
 		memcpy (&data, p, sizeof (data));
 		return data;
 	} else {
-		g_error ("Little endian machine, but weird size of doubles");
+		g_error ("Little endian machine, but weird size of guint64");
 	}
 #else
 #error "Byte order not recognised -- out of luck"
@@ -135,7 +166,7 @@ gsf_le_get_guint64 (void const *p)
 float
 gsf_le_get_float (void const *p)
 {
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (float) == 4) {
 		float   f;
 		int     i;
@@ -148,9 +179,9 @@ gsf_le_get_float (void const *p)
 
 		return f;
 	} else {
-		g_error ("Big endian machine, but weird size of float");
+		g_error ("Big endian machine, but weird size of floats");
 	}
-#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+#elif (G_FLOAT_BYTE_ORDER == G_LITTLE_ENDIAN) || (G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN)
 	if (sizeof (float) == 4) {
 		/*
 		 * On i86, we could access directly, but Alphas require
@@ -160,17 +191,17 @@ gsf_le_get_float (void const *p)
 		memcpy (&data, p, sizeof (data));
 		return data;
 	} else {
-		g_error ("Little endian machine, but weird size of doubles");
+		g_error ("Little endian machine, but weird size of floats");
 	}
 #else
-#error "Byte order not recognised -- out of luck"
+#error "Floating-point byte order not recognised -- out of luck"
 #endif
 }
 
 void
 gsf_le_set_float (void *p, float d)
 {
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (float) == 4) {
 		int     i;
 		guint8 *t  = (guint8 *)&d;
@@ -180,9 +211,9 @@ gsf_le_set_float (void *p, float d)
 		for (i = 0; i < sd; i++)
 			p2[sd - 1 - i] = t[i];
 	} else {
-		g_error ("Big endian machine, but weird size of doubles");
+		g_error ("Big endian machine, but weird size of floats");
 	}
-#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+#elif (G_FLOAT_BYTE_ORDER == G_LITTLE_ENDIAN) || (G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN)
 	if (sizeof (float) == 4) {
 		/*
 		 * On i86, we could access directly, but Alphas require
@@ -190,17 +221,22 @@ gsf_le_set_float (void *p, float d)
 		 */
 		memcpy (p, &d, sizeof (d));
 	} else {
-		g_error ("Little endian machine, but weird size of doubles");
+		g_error ("Little endian machine, but weird size of floats");
 	}
 #else
-#error "Byte order not recognised -- out of luck"
+#error "Floating-point byte order not recognised -- out of luck"
 #endif
 }
 
 double
 gsf_le_get_double (void const *p)
 {
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN
+	double data;
+	memcpy ((char *)&data + 4, p, 4);
+	memcpy ((char *)&data, (const char *)p + 4, 4);
+	return data;
+#elif G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (double) == 8) {
 		double  d;
 		int     i;
@@ -215,7 +251,7 @@ gsf_le_get_double (void const *p)
 	} else {
 		g_error ("Big endian machine, but weird size of doubles");
 	}
-#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+#elif G_FLOAT_BYTE_ORDER == G_LITTLE_ENDIAN
 	if (sizeof (double) == 8) {
 		/*
 		 * On i86, we could access directly, but Alphas require
@@ -228,14 +264,17 @@ gsf_le_get_double (void const *p)
 		g_error ("Little endian machine, but weird size of doubles");
 	}
 #else
-#error "Byte order not recognised -- out of luck"
+#error "Floating-point byte order not recognised -- out of luck"
 #endif
 }
 
 void
 gsf_le_set_double (void *p, double d)
 {
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if G_FLOAT_BYTE_ORDER == G_ARMFLOAT_ENDIAN
+	memcpy (p, (const char *)&d + 4, 4);
+	memcpy ((char *)p + 4, &d, 4);
+#elif G_FLOAT_BYTE_ORDER == G_BIG_ENDIAN
 	if (sizeof (double) == 8) {
 		int     i;
 		guint8 *t  = (guint8 *)&d;
@@ -247,7 +286,7 @@ gsf_le_set_double (void *p, double d)
 	} else {
 		g_error ("Big endian machine, but weird size of doubles");
 	}
-#elif G_BYTE_ORDER == G_LITTLE_ENDIAN
+#elif G_FLOAT_BYTE_ORDER == G_LITTLE_ENDIAN
 	if (sizeof (double) == 8) {
 		/*
 		 * On i86, we could access directly, but Alphas require
@@ -258,7 +297,7 @@ gsf_le_set_double (void *p, double d)
 		g_error ("Little endian machine, but weird size of doubles");
 	}
 #else
-#error "Byte order not recognised -- out of luck"
+#error "Floating-point byte order not recognised -- out of luck"
 #endif
 }
 
@@ -376,6 +415,9 @@ static char const *base64_alphabet =
 
 #define d(x)
 
+/* Line length for base64 encoding.  Must be a multiple of 4. */
+enum { BASE64_LINE_LEN = 76 };
+
 static void
 base64_init(void)
 {
@@ -420,9 +462,10 @@ gsf_base64_encode_close (guint8 const *in, size_t inlen,
 		outptr[1] = base64_alphabet[ c2 >> 4 | ( (c1&0x3) << 4 )];
 		outptr[3] = '=';
 		outptr += 4;
+		++*state;
 		break;
 	}
-	if (break_lines)
+	if (break_lines && *state > 0)
 		*outptr++ = '\n';
 
 	*save = 0;
@@ -476,7 +519,7 @@ gsf_base64_encode_step (guint8 const *in, size_t len,
 			*outptr++ = base64_alphabet[ ( (c2 &0x0f) << 2 ) | (c3 >> 6) ];
 			*outptr++ = base64_alphabet[ c3 & 0x3f ];
 			/* this is a bit ugly ... */
-			if (break_lines && (++already)>=59) {
+			if (break_lines && (++already) * 4 >= BASE64_LINE_LEN) {
 				*outptr++='\n';
 				already = 0;
 			}
@@ -582,9 +625,14 @@ gsf_base64_encode_simple (guint8 const *data, size_t len)
 	guint8 *out;
 	int state = 0, outlen;
 	unsigned int save = 0;
-	
-	out = g_new (guint8, len * 4 / 3 + 5);
-	outlen = gsf_base64_encode_close (data, len, FALSE, out, &state, &save);
+	gboolean break_lines = TRUE;
+
+	outlen = len * 4 / 3 + 5;
+	if (break_lines)
+		outlen += outlen / BASE64_LINE_LEN + 1;
+	out = g_new (guint8, outlen);
+	outlen = gsf_base64_encode_close (data, len, break_lines,
+					  out, &state, &save);
 	out [outlen] = '\0';
 	return out;
 }
